@@ -1,9 +1,11 @@
 #include <stdlib.h>
 #include <iostream>
 
+#include <kinstance.h>
+
 #include <qptrlist.h>
-#include "kstfile.h"
 #include "kstrvector.h"
+#include "kstdatasource.h"
 
 void Usage() {
   fprintf(stderr, "usage: d2asc filename [-f <first frame>]\n");
@@ -14,7 +16,8 @@ void Usage() {
 }
 
 int main(int argc, char *argv[]) {
-  KstFilePtr file;
+  KInstance inst("d2asc");
+  KstDataSourcePtr file;
   int i;
 
   char field_list[40][120], filename[180];
@@ -62,8 +65,8 @@ int main(int argc, char *argv[]) {
 
   if (!do_skip) do_ave = false;
 
-  file = new KstFile(filename);
-  if (file->numFrames() < 1) {
+  file = KstDataSource::loadSource(filename);
+  if (!file || !file->isValid() || file->frameCount() < 1) {
     fprintf(stderr, "d2asc error: file %s has no data\n", filename);
     exit(0);
   }
