@@ -112,7 +112,8 @@ KstObject::UpdateType AsciiSource::update(int u) {
         if (!is_comment) {
           has_dat = true;
         }
-      } else if (tmpbuf[i] == '\n') {
+      } else if (tmpbuf[i] == '\n' ||
+                 tmpbuf[i] == '\r') {
         if (has_dat) {
           _numFrames++;
           if (_numFrames >= _numLinesAlloc) {
@@ -169,21 +170,26 @@ int AsciiSource::readField(double *v, const QString& field, int s, int n) {
   file.at(bufstart);
   file.readBlock(tmpbuf, bufread);
 
+  memset( v, 0, n * sizeof( double ) );
+  
   for (int i = 0; i < n; i++, s++) {
     bool done = false;
     bool incol = false;
     int i_col = 0;
-    for (int ch = _rowIndex[s] - bufstart; !done; ch++) {
+    for (int ch = _rowIndex[s] - bufstart; !done && ch < bufread; ch++) {
       if (isspace(tmpbuf[ch])) {
-        incol = false;
-      } else if (tmpbuf[ch] == '\n') {
-        done = true;
-        v[i] = 0;
+        if (tmpbuf[ch] == '\n' ||
+            tmpbuf[ch] == '\r' ) {
+          done = true;
+        }
+        else
+        {
+          incol = false;
+        }
       } else if (tmpbuf[ch] == '#' || tmpbuf[ch] == '!' ||
           tmpbuf[ch] == '/' || tmpbuf[ch] == ';' ||
           tmpbuf[ch] == 'c') {
         done = true;
-        v[i] = 0;
       } else {
         if (!incol) {
           incol = true;
