@@ -1380,7 +1380,9 @@ int GetNFrames(const char *filename_in, int *error_code, const char *in_field) {
   else field = in_field;
     
   sprintf(raw_data_filename,"%s/%s", filename, field);
-  stat(raw_data_filename, &statbuf);
+  if (stat(raw_data_filename, &statbuf) < 0) {
+    return(0);
+  }
 
   nf = statbuf.st_size/
 	 (F->rawEntries[0].size*F->rawEntries[0].samples_per_frame);
