@@ -27,12 +27,14 @@
 // include files for KDE
 #include <kaccel.h>
 #include <kconfig.h>
+#include <kdebug.h>
 #include <kfiledialog.h>
 #include <kiconloader.h>
 #include <kkeydialog.h>
 #include <klocale.h>
 #include <kmenubar.h>
 #include <kmessagebox.h>
+#include <kparts/componentfactory.h>
 #include <kprinter.h>
 #include <kstandarddirs.h>
 #include <kstatusbar.h>
@@ -44,7 +46,9 @@
 #include "kst.h"
 #include "kstview.h"
 #include "kstdoc.h"
+#include "kstextension.h"
 #include "kstplotdialog_i.h"
+#include "kstviewscalarsdialog_i.h"
 #include "kstchangefiledialog_i.h"
 #include "kstchangenptsdialog_i.h"
 #include "kstquickcurvesdialog_i.h"
@@ -83,6 +87,7 @@ KstApp::KstApp(QWidget *parent, const char* name)
   plotDialog = new KstPlotDialogI(doc, this);
   dataManager = new KstDataManagerI(doc, this);
   filterListEditor = new FilterListEditor(this);
+  //viewScalarsDialog = new KstViewScalarsDialogI(this);
   changeFileDialog = new KstChangeFileDialogI(this);
   changeNptsDialog = new KstChangeNptsDialogI(this);
   quickCurvesDialog = new KstQuickCurvesDialogI(this);
@@ -148,6 +153,23 @@ KstApp::KstApp(QWidget *parent, const char* name)
   connect(this, SIGNAL(settingsChanged()), this, SLOT(slotSettingsChanged()));
 
   QTimer::singleShot(0, this, SLOT(updateActions()));
+
+  // Load any extensions
+  KService::List sl = KServiceType::offers("Kst Extension");
+  for (KService::List::ConstIterator it = sl.begin(); it != sl.end(); ++it) {
+    KService::Ptr service = *it;
+    kdDebug() << "Found Kst Extension " << service->property("Name").toString() << endl;
+    int err = 0;
+    KstExtension *e = KParts::ComponentFactory::createInstanceFromService<KstExtension>(service, this, 0, QStringList(), &err);
+    if (e) {
+      kdDebug() << "Kst Extension " << service->property("Name").toString() << " loaded" << endl;
+    } else {
+      kdDebug() << "Error trying to load extension: " << err << endl;
+      if (err == KParts::ComponentFactory::ErrNoLibrary) {
+        kdDebug() << "Library error: " << KLibLoader::self()->lastErrorMessage() << endl;
+      }
+    }
+  }
 }
 
 void KstApp::updateActions() {
@@ -211,6 +233,9 @@ void KstApp::initActions() {
   StatusBarAction->setWhatsThis(i18n("Toggle Statusbar"));
   connect(StatusBarAction, SIGNAL(activated()), this, SLOT(setSettingsDirty()));
 
+  /************/
+  KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
+
   /************/
   recent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL &)),
                            actionCollection());
@@ -341,6 +366,17 @@ void KstApp::initActions() {
                                             "to change input files."));
 
   /************/
+#if 0
+  ViewScalarsDialogAction = new KAction(i18n("View &Scalars"),
+                                       0, 0, this,
+                                       SLOT(showViewScalarsDialog()),
+                                       actionCollection(),
+                                       "viewscalarsdialog_action");
+  ViewScalarsDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
+                                            "to view scalar values."));
+#endif
+  
+  /************/
   ChangeNptsDialogAction = new KAction(i18n("Change Data Sample &Ranges"),
                                        "kst_changenpts", 0, this,
                                        SLOT(showChangeNptsDialog()),
@@ -799,6 +835,10 @@ void KstApp::showDataManager() {
   dataManager->show_I();
 }
 
+void KstApp::showViewScalarsDialog() {
+  //viewScalarsDialog->showViewScalarsDialog();
+}
+
 void KstApp::showChangeFileDialog() {
   changeFileDialog->showChangeFileDialog();
 }
@@ -853,6 +893,11 @@ void KstApp::updateDialogs() {
   quickPSDDialog->update();
   dataManager->update();
   filterListEditor->update();
+#if 0
+  if (!viewScalarsDialog->isHidden()) {
+    viewScalarsDialog->updateViewScalarsDialog();
+  }
+#endif
   vectorSaveDialog->init();
   view->update();
 }
