@@ -140,7 +140,7 @@ void KstCurveDialogI::update(int new_index) {
 
   if (isNew) {
     QString new_label;
-    new_label.sprintf("C%d-", curves.count()+1);
+    new_label.sprintf("C%d-", signed(curves.count())+1);
     new_label += i18n("<New_Curve>");
 
     Select->insertItem(new_label);
@@ -222,8 +222,10 @@ void KstCurveDialogI::new_I() {
   tag_name.replace(i18n("<New_Curve>"), (*VY)->tagName());
 
   /* verify that the curve name is unique */
-  if (KST::dataTagNameNotUnique(tag_name))
+  if (KST::dataTagNameNotUnique(tag_name)) {
+    Select->setFocus();
     return;
+  }
 
   /* create the curve */
   curve = new KstVCurve(tag_name, *VX, *VY,
@@ -268,8 +270,10 @@ void KstCurveDialogI::edit_I() {
 
   /* verify that the curve name is unique */
   if (tag_name != curves[index]->tagName()) {
-    if (KST::dataTagNameNotUnique(tag_name))
+    if (KST::dataTagNameNotUnique(tag_name)) {
+      Select->setFocus();
       return;
+    }
   }
 
   curve = curves[index];
@@ -323,3 +327,4 @@ void KstCurveDialogI::delete_I() {
 }
 
 #include "kstcurvedialog_i.moc"
+//vim: ts=2 sw=2 et
