@@ -29,3 +29,6 @@
   /** The list of data objects which are in use */
   KstDataObjectList KST::dataObjectList;
 
+  /** The list of filter sets defined */
+  KstFilterSetList KST::filterSetList;
+
