@@ -43,11 +43,12 @@ namespace KST {
   extern KstFilterSetList filterSetList;
 
   /** check that a tag has not been used by any other tags */
-  extern bool dataTagNameNotUnique(const QString &tag, bool warn = true);
-  extern bool vectorTagNameNotUnique(const QString &tag, bool warn = true);
+  extern bool tagNameNotUnique(const QString &tag, bool warn = true, void *parent = 0L);
+  extern bool dataTagNameNotUnique(const QString &tag, bool warn = true, void *parent = 0L);
+  extern bool vectorTagNameNotUnique(const QString &tag, bool warn = true, void *parent = 0L);
 
   /** Save a vector to a file */
-  extern int vectorToFile(KstVectorPtr v, QFile *f);
+  extern int vectorToFile(KstVectorPtr v, QFile *f, void *parent = 0L);
 }
 
 #endif
