@@ -16,9 +16,11 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "kstdataobject.h"
 #include "kstdatacollection.h"
+#include "kstdataobject.h"
+
 #include <qtimer.h>
+
 #include <kdebug.h>
 
 KstDataObject::KstDataObject() : KstObject() {
@@ -26,7 +28,7 @@ KstDataObject::KstDataObject() : KstObject() {
 }
 
 KstDataObject::KstDataObject(QDomElement& e) : KstObject() {
-Q_UNUSED(e);
+Q_UNUSED(e)
   //kdDebug() << "+++ CREATING DATA OBJECT: " << (void*)this << endl;
 }
 
