@@ -90,6 +90,10 @@ namespace KST {
 
         QCString libname = QFile::encodeName(QString("kstdata_") + _plugLib);
         _lib = KLibLoader::self()->library(libname);
+        if (!_lib) {
+          kdDebug() << "Error datasource plugin [" << libname << "]" << endl;
+          kdDebug() << "Reason: " << KLibLoader::self()->lastErrorMessage() << endl;
+        }
         return _lib != 0L;
       }
 
