@@ -65,20 +65,21 @@ static bool backupFile(const QString& qFilename, const QString& backupDir = QStr
 
 KstDoc::KstDoc(QWidget *parent, const char *name)
 : QObject(parent, name) {
-    stopping = false;
-    UpdateCounter = 0;
-    _updateDelay = 200;
+  _lock = 0;
+  stopping = false;
+  UpdateCounter = 0;
+  _updateDelay = 200;
 }
 
 KstDoc::~KstDoc() {
-    // We -have- to clear this here because static destruction order is not
-    // guaranteed by any means and KstDoc::vectorList is static and has a
-    // pointer to KstDoc::scalarList, also static, which gets used during
-    // vector destruction.
-    KST::filterSetList.clear();
-    KST::plotList.clear();
-    KST::vectorList.clear();
-    KST::scalarList.clear();
+  // We -have- to clear this here because static destruction order is not
+  // guaranteed by any means and KstDoc::vectorList is static and has a
+  // pointer to KstDoc::scalarList, also static, which gets used during
+  // vector destruction.
+  KST::filterSetList.clear();
+  KST::plotList.clear();
+  KST::vectorList.clear();
+  KST::scalarList.clear();
 }
 
 void KstDoc::setAbsFilePath(const QString &filename) {
@@ -213,6 +214,8 @@ bool KstDoc::openDocument(const KURL &url, const QString &o_file,
         if (file) {
           KST::dataSourceList.append(file);
         }
+      } else if (e.tagName() == "scalar") {
+        new KstScalar(e);
       } else if (e.tagName() == "vector") {
         vector = new KstRVector(e, o_file, o_n, o_f, o_s, o_ave);
         // Vectors are automatically added to the global list.
@@ -292,6 +295,15 @@ bool KstDoc::saveDocument(const QString &filename) {
     ts << " </kstfile>" << endl;
   }
 
+  // save orphan scalars
+  for (KstScalarList::Iterator it = KST::scalarList.begin(); it != KST::scalarList.end(); ++it) {
+    if ((*it)->orphan()) {
+      ts << " <scalar>" << endl;
+      (*it)->save(ts);
+      ts << " </scalar>" << endl;
+    }
+  }
+
   // save vectors
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
@@ -349,6 +361,15 @@ void KstDoc::deleteContents()
   KST::dataSourceList.clear();
 }
 
+
+class FakeLocker {
+  public:
+    FakeLocker(int *x) : _x(x) { *_x = 1; }
+    ~FakeLocker() { *_x = 0; }
+  private:
+    int *_x;
+};
+
 /*****************
     Return values:
     NO_CHANGE: Nothing changed and so no view updates are required
@@ -370,10 +391,18 @@ void KstDoc::deleteContents()
 /****/
 KstObject::UpdateType KstDoc::update(bool force) {
   unsigned i;
-  int lastFrame=-1;
+  int lastFrame = -1;
   QString message;
   KstObject::UpdateType tU, U = KstObject::NO_CHANGE;
 
+  // Prevent reentrancy due to event processing!!
+  if (_lock != 0) {
+    kdDebug() << "LOCKED" << endl;
+    return U;
+  }
+
+  FakeLocker ml(&_lock);
+
   UpdateCounter++;
   if (UpdateCounter < 1) {
     UpdateCounter = 1; // check for wrap around
@@ -419,8 +448,12 @@ KstObject::UpdateType KstDoc::update(bool force) {
   }
 
   /* update all vectors  -  brute force */
-  for (KstVectorList::Iterator it = KST::vectorList.begin(); it != KST::vectorList.end(); ++it) {
-    if (dynamic_cast<KstRVector*>((*it).data())) {
+  // FIXME: can't update slave vectors, so for now we just check for
+  //        filtered vectors - yuck
+  KstVectorList vl = KST::vectorList;
+  for (KstVectorList::Iterator it = vl.begin(); it != vl.end(); ++it) {
+    //if (dynamic_cast<KstRVector*>((*it).data())) {
+    if (!dynamic_cast<KstFilteredVector*>((*it).data())) {
       continue;
     }
 
@@ -430,7 +463,7 @@ KstObject::UpdateType KstDoc::update(bool force) {
       if (stopping) {
         return KstObject::NO_CHANGE;
       }
-      if (tU == KstObject::READ) {
+      if (tU == KstObject::READ && U != KstObject::UPDATE) {
         U = KstObject::READ;
       } else if (U == KstObject::NO_CHANGE) {
         U = KstObject::UPDATE;
@@ -440,7 +473,8 @@ KstObject::UpdateType KstDoc::update(bool force) {
 
   // update curves next
   if ((U == KstObject::UPDATE) || force) {
-    for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
+    KstDataObjectList dol = KST::dataObjectList;
+    for (KstDataObjectList::Iterator it = dol.begin(); it != dol.end(); ++it) {
       KstBaseCurve *bc = dynamic_cast<KstBaseCurve*>((*it).data());
       if (bc) {
         bc->update(UpdateCounter);
