@@ -204,7 +204,10 @@ void KstEqDialogI::new_I() {
   tag_name.replace(i18n("<New_Equation>"), Equation->text());
 
   /* verify that the curve name is unique */
-  if (KST::dataTagNameNotUnique(tag_name)) return;
+  if (KST::dataTagNameNotUnique(tag_name)) {
+    Select->setFocus();
+    return;
+  }
 
   if (UseExistingVector->isChecked()) {
     /* find *V */
@@ -297,8 +300,10 @@ void KstEqDialogI::edit_I() {
   QString tag_name = Select->currentText();
 
   if (tag_name != eq->tagName()) {
-    if (KST::dataTagNameNotUnique(tag_name))
+    if (KST::dataTagNameNotUnique(tag_name)) {
+      Select->setFocus();
       return;
+    }
   }
   eq->setTagName(tag_name);
 
@@ -368,3 +373,4 @@ void KstEqDialogI::delete_I() {
 }
 
 #include "ksteqdialog_i.moc"
+// vim: ts=2 sw=2 et
