@@ -15,7 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/** A class for handling power spectra for kst
+/** A class for handling equations for kst
  *@author C. Barth Netterfield
  */
 #ifndef KSTEQUATIONCURVE_H
@@ -26,11 +26,14 @@
 #include "kstscalar.h"
 #include <qstring.h>
 #include <qcolor.h>
-#include "genterStacks.h"
 
 #define MAX_DIV_REG 100
 
-class KstEquationCurve: public KstBaseCurve {
+namespace Equation {
+  class Node;
+}
+
+class KstEquationCurve : public KstBaseCurve {
 public:
   KstEquationCurve(const QString &in_tag, const QString &equation,
                    double x0, double x1, int nx,
@@ -82,19 +85,10 @@ private:
   bool _staticX;
 
   KstVectorList VectorsUsed;
-  KstScalarList ScalarsUsed;
 
   void commonConstructor(const QString &in_tag, const QString &equation,
                          const QColor &in_color);
 
-  void preProcess();
-
-  ValStack vstack;
-  OpStack opstack;
-  NumStack numstack;
-  GQ gq;
-
-  bool FillRPNQ();
   bool FillY(bool force = false);
   bool IsValid;
 
@@ -103,10 +97,10 @@ private:
   bool DoInterp;
   int Interp;
 
-  int DivRegPtr;
-  double DivReg[MAX_DIV_REG];
   static const QString XVECTOR;
   static const QString OUTVECTOR;
+  KstVectorMap::Iterator _xVector, _yVector;
+  Equation::Node *_pe;
 };
 
 typedef KSharedPtr<KstEquationCurve> KstEquationCurvePtr;
