@@ -105,7 +105,7 @@ void KstHistogram::commonConstructor(const QString &in_tag, KstVectorPtr in_V,
 
   NormMode = in_norm_mode;
 
-  _tag = in_tag;
+  setTagName(in_tag);
   V = in_V;
 
   if (xmax_in>xmin_in) {
@@ -283,33 +283,33 @@ KstCurveType KstHistogram::type() const {
 }
 
 void KstHistogram::save(QTextStream &ts) {
-  ts << " <histogram>\n";
-  ts << "  <tag>" << _tag << "</tag>\n";
-  ts << "  <vectag>" << V->tagName() << "</vectag>\n";
+  ts << " <histogram>" << endl;
+  ts << "  <tag>" << tagName() << "</tag>" << endl;
+  ts << "  <vectag>" << V->tagName() << "</vectag>" << endl;
 
-  ts << "  <color>" << Color.name() << "</color>\n";
-  ts << "  <hasLines>" << HasLines << "</hasLines>\n";
-  ts << "  <hasPoints>" << HasPoints << "</hasPoints>\n";
-  ts << "  <pointType>" << Point.getType() << "</pointType>\n";
+  ts << "  <color>" << Color.name() << "</color>" << endl;
+  ts << "  <hasLines>" << HasLines << "</hasLines>" << endl;
+  ts << "  <hasPoints>" << HasPoints << "</hasPoints>" << endl;
+  ts << "  <pointType>" << Point.getType() << "</pointType>" << endl;
 
-  ts << "  <numBins>"  << NBins << "</numBins>\n";
-  ts << "  <minX>" << MinX << "</minX>\n";
-  ts << "  <maxX>" << MaxX << "</maxX>\n";
+  ts << "  <numBins>"  << NBins << "</numBins>" << endl;
+  ts << "  <minX>" << MinX << "</minX>" << endl;
+  ts << "  <maxX>" << MaxX << "</maxX>" << endl;
   switch (NormMode) {
   case KST_HS_NUMBER:
-    ts << "  <NormMode>NUMBER</NormMode>\n";
+    ts << "  <NormMode>NUMBER</NormMode>" << endl;
     break;
   case KST_HS_PERCENT:
-    ts << "  <NormMode>PERCENT</NormMode>\n";
+    ts << "  <NormMode>PERCENT</NormMode>" << endl;
     break;
   case KST_HS_FRACTION:
-    ts << "  <NormMode>FRACTION</NormMode>\n";
+    ts << "  <NormMode>FRACTION</NormMode>" << endl;
     break;
   case KST_HS_MAX_ONE:
-    ts << "  <NormMode>MAX_ONE</NormMode>\n";
+    ts << "  <NormMode>MAX_ONE</NormMode>" << endl;
     break;
   }
-  ts << " </histogram>\n";
+  ts << " </histogram>" << endl;
 }
 
 double KstHistogram::getVMax() const {
