@@ -199,8 +199,10 @@ void KstHsDialogI::new_I() {
   tag_name.replace(i18n("<New_Histogram>"), "HS_" + _vector->selectedVector());
 
   /* verify that the curve name is unique */
-  if (KST::dataTagNameNotUnique(tag_name))
+  if (KST::dataTagNameNotUnique(tag_name)) {
+    Select->setFocus();
     return;
+  }
 
   if (_vector->selectedVector().isEmpty()) {
     KMessageBox::sorry(0L, i18n("New Histogram not made: define vectors first."));
@@ -287,6 +289,7 @@ void KstHsDialogI::edit_I() {
   hs = curves[index];
   if (tag_name != hs->tagName()) {
     if (KST::dataTagNameNotUnique(tag_name)) {
+      Select->setFocus();
       return;
     }
   }
@@ -385,3 +388,4 @@ void KstHsDialogI::autoBin() {
 }
 
 #include "ksthsdialog_i.moc"
+// vim: ts=2 sw=2 et
