@@ -33,6 +33,7 @@
 
 #include <qfile.h>
 #include <qfileinfo.h>
+#include <qregexp.h>
 
 
 KstIfaceImpl::KstIfaceImpl(KstDoc *doc, KstApp *app)
@@ -66,7 +67,7 @@ QStringList KstIfaceImpl::objectList() {
 
 
 QStringList KstIfaceImpl::curveList() {
-QStringList rc;
+  QStringList rc;
 
   KstBaseCurveList bcl = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
 
@@ -74,12 +75,12 @@ QStringList rc;
     rc += (*it)->tagName();
   }
 
-return rc;
+  return rc;
 }
 
 
 QStringList KstIfaceImpl::plotList() {
-return KST::plotList.tagNames();
+  return KST::plotList.tagNames();
 }
 
 
@@ -148,17 +149,15 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
     return false;
   }
 
-  // FIXME: generate etag, ptag properly - work out semantics for finding existing plot
-  etag = "Eq-" + plotName;
+  etag = "Eq-" + QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_");
   ptag = "P-" + plotName;
 
   if (!plotName.isEmpty()) {
     plot = KST::plotList.FindKstPlot(plotName);
   }
 
-  if (!plot) {
-    plot = new KstPlot(ptag);
-    KST::plotList.append(plot);
+  while (KST::dataTagNameNotUnique(etag, false)) {
+    etag += '\'';
   }
 
   KstEquationCurvePtr eq;
@@ -166,6 +165,15 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
   eq = new KstEquationCurve(etag, equation, start, end, numSamples,
                             color.isValid() ? color : QColor("darkBlue"));
 
+  if (!eq->isValid()) {
+    return false;
+  }
+
+  if (!plot) {
+    plot = new KstPlot(ptag);
+    KST::plotList.append(plot);
+  }
+
   KST::dataObjectList.append(KstDataObjectPtr(eq));
 
   plot->Curves.append(KstBaseCurvePtr(eq));
@@ -201,17 +209,15 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
 
   v = *it;
 
-  // FIXME: generate etag, ptag properly - work out semantics for finding existing plot
-  etag = "Eq-" + plotName;
+  etag = "Eq-" + QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_");
   ptag = "P-" + plotName;
 
   if (!plotName.isEmpty()) {
     plot = KST::plotList.FindKstPlot(plotName);
   }
 
-  if (!plot) {
-    plot = new KstPlot(ptag);
-    KST::plotList.append(plot);
+  while (KST::dataTagNameNotUnique(etag, false)) {
+    etag += '\'';
   }
 
   KstEquationCurvePtr eq;
@@ -219,6 +225,15 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
   eq = new KstEquationCurve(etag, equation, v, true,
                             color.isValid() ? color : QColor("darkBlue"));
 
+  if (!eq->isValid()) {
+    return false;
+  }
+
+  if (!plot) {
+    plot = new KstPlot(ptag);
+    KST::plotList.append(plot);
+  }
+
   KST::dataObjectList.append(KstDataObjectPtr(eq));
 
   plot->Curves.append(KstBaseCurvePtr(eq));
@@ -234,14 +249,15 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
 
 
 const QString& KstIfaceImpl::generateScalar(const QString& name, double value) {
-KstScalarPtr s = new KstScalar(name, value);
-return s->tagName();
+  KstScalarPtr s = new KstScalar(name, value);
+  s->setOrphan(true);
+  return s->tagName();
 }
 
 
 const QString& KstIfaceImpl::generateVector(const QString& name, double from, double to, int points) {
-KstVectorPtr v = KstVector::generateVector(from, to, points, name);
-return v->tagName();
+  KstVectorPtr v = KstVector::generateVector(from, to, points, name);
+  return v->tagName();
 }
 
 
@@ -286,8 +302,8 @@ QStringList rc;
 
 
 QStringList KstIfaceImpl::outputVectors(const QString& objectName) {
-KstDataObjectList::Iterator oi = KST::dataObjectList.findTag(objectName);
-QStringList rc;
+  KstDataObjectList::Iterator oi = KST::dataObjectList.findTag(objectName);
+  QStringList rc;
 
   if (oi != KST::dataObjectList.end()) {
     rc = (*oi)->outputVectors().tagNames();
@@ -298,8 +314,8 @@ QStringList rc;
 
 
 QStringList KstIfaceImpl::outputScalars(const QString& objectName) {
-KstDataObjectList::Iterator oi = KST::dataObjectList.findTag(objectName);
-QStringList rc;
+  KstDataObjectList::Iterator oi = KST::dataObjectList.findTag(objectName);
+  QStringList rc;
 
   if (oi != KST::dataObjectList.end()) {
     rc = (*oi)->outputScalars().tagNames();
@@ -310,7 +326,7 @@ QStringList rc;
 
 
 double KstIfaceImpl::scalar(const QString& name) {
-KstScalarList::Iterator it = KST::scalarList.findTag(name);
+  KstScalarList::Iterator it = KST::scalarList.findTag(name);
 
   if (it == KST::scalarList.end()) {
     return 0.0;
@@ -321,7 +337,7 @@ KstScalarList::Iterator it = KST::scalarList.findTag(name);
 
 
 bool KstIfaceImpl::setScalar(const QString& name, double value) {
-KstScalarList::Iterator it = KST::scalarList.findTag(name);
+  KstScalarList::Iterator it = KST::scalarList.findTag(name);
 
   if (it == KST::scalarList.end()) {
     return false;
@@ -333,67 +349,67 @@ KstScalarList::Iterator it = KST::scalarList.findTag(name);
 
 
 int KstIfaceImpl::vectorSize(const QString& name) {
-KstVectorList::Iterator it = KST::vectorList.findTag(name);
-int rc = 0;
+  KstVectorList::Iterator it = KST::vectorList.findTag(name);
+  int rc = 0;
 
   if (it != KST::vectorList.end()) {
     rc = (*it)->length();
   }
 
-return rc;
+  return rc;
 }
 
 
 double KstIfaceImpl::vector(const QString& name, int index) {
-KstVectorList::Iterator it = KST::vectorList.findTag(name);
-double rc = 0.0;
+  KstVectorList::Iterator it = KST::vectorList.findTag(name);
+  double rc = 0.0;
 
-  if (it != KST::vectorList.end() && rc >= 0 && rc < (*it)->length()) {
+  if (it != KST::vectorList.end() && index >= 0 && index < (*it)->length()) {
     rc = (*it)->value(index);
   }
 
-return rc;
+  return rc;
 }
 
 
 bool KstIfaceImpl::setVector(const QString& name, int index, double value) {
-KstVectorList::Iterator it = KST::vectorList.findTag(name);
+  KstVectorList::Iterator it = KST::vectorList.findTag(name);
 
   if (it != KST::vectorList.end() && index >= 0 && index < (*it)->length()) {
     (*it)->value()[index] = value;
     return true;
   }
 
-return false;
+  return false;
 }
 
 
 bool KstIfaceImpl::resizeVector(const QString& name, int newSize) {
-KstVectorList::Iterator it = KST::vectorList.findTag(name);
+  KstVectorList::Iterator it = KST::vectorList.findTag(name);
 
   if (it != KST::vectorList.end()) {
     (*it)->resize(newSize);
     return (*it)->length() == newSize;
   }
 
-return false;
+  return false;
 }
 
 
 bool KstIfaceImpl::clearVector(const QString& name) {
-KstVectorList::Iterator it = KST::vectorList.findTag(name);
+  KstVectorList::Iterator it = KST::vectorList.findTag(name);
 
   if (it != KST::vectorList.end()) {
     (*it)->zero();
     return true;
   }
 
-return false;
+  return false;
 }
 
 
 const QString& KstIfaceImpl::createPlot(const QString& name) {
-QString n = name;
+  QString n = name;
 
   if (KST::plotList.FindKstPlot(n)) {
     n = KST::plotList.generatePlotName();
@@ -406,12 +422,12 @@ QString n = name;
   _doc->updateDialogs();
   KST::plotList.arrangePlots(KST::plotList.getPlotCols());
 
-return p->tagName();
+  return p->tagName();
 }
 
 
 bool KstIfaceImpl::deletePlot(const QString& name) {
-KstPlot *p = KST::plotList.FindKstPlot(name);
+  KstPlot *p = KST::plotList.FindKstPlot(name);
 
   if (p) {
     KST::plotList.remove(p);
@@ -422,12 +438,12 @@ KstPlot *p = KST::plotList.FindKstPlot(name);
     return true;
   }
 
-return false;
+  return false;
 }
 
 
 QStringList KstIfaceImpl::plotContents(const QString& name) {
-KstPlot *p = KST::plotList.FindKstPlot(name);
+  KstPlot *p = KST::plotList.FindKstPlot(name);
 
   if (!p) {
     return QStringList();
@@ -438,9 +454,9 @@ KstPlot *p = KST::plotList.FindKstPlot(name);
 
 
 bool KstIfaceImpl::addCurveToPlot(const QString& plot, const QString& curve) {
-KstPlot *p = KST::plotList.FindKstPlot(plot);
-KstBaseCurveList bcl = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
-KstBaseCurveList::Iterator ci = bcl.findTag(curve);
+  KstPlot *p = KST::plotList.FindKstPlot(plot);
+  KstBaseCurveList bcl = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
+  KstBaseCurveList::Iterator ci = bcl.findTag(curve);
 
   if (p && ci != bcl.end()) {
     if (!p->Curves.contains(*ci)) {
@@ -452,14 +468,14 @@ KstBaseCurveList::Iterator ci = bcl.findTag(curve);
     return true;
   }
 
-return false;
+  return false;
 }
 
 
 bool KstIfaceImpl::removeCurveFromPlot(const QString& plot, const QString& curve) {
-KstPlot *p = KST::plotList.FindKstPlot(plot);
-KstBaseCurveList bcl = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
-KstBaseCurveList::Iterator ci = bcl.findTag(curve);
+  KstPlot *p = KST::plotList.FindKstPlot(plot);
+  KstBaseCurveList bcl = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
+  KstBaseCurveList::Iterator ci = bcl.findTag(curve);
 
   if (p && ci != bcl.end()) {
     p->Curves.remove(*ci);
@@ -469,7 +485,7 @@ KstBaseCurveList::Iterator ci = bcl.findTag(curve);
     return true;
   }
 
-return false;
+  return false;
 }
 
 
@@ -479,11 +495,11 @@ const QString& KstIfaceImpl::createCurve(const QString& name, const QString& xVe
 
 
 const QString& KstIfaceImpl::createCurve(const QString& name, const QString& xVector, const QString& yVector, const QString& xErrorVector, const QString& yErrorVector, const QColor& color) {
-QString n = name;
-KstVectorList::Iterator vx = KST::vectorList.findTag(xVector);
-KstVectorList::Iterator vy = KST::vectorList.findTag(yVector);
-KstVectorList::Iterator ex = KST::vectorList.findTag(xErrorVector);
-KstVectorList::Iterator ey = KST::vectorList.findTag(yErrorVector);
+  QString n = name;
+  KstVectorList::Iterator vx = KST::vectorList.findTag(xVector);
+  KstVectorList::Iterator vy = KST::vectorList.findTag(yVector);
+  KstVectorList::Iterator ex = KST::vectorList.findTag(xErrorVector);
+  KstVectorList::Iterator ey = KST::vectorList.findTag(yErrorVector);
 
   while (KST::dataObjectList.findTag(n) != KST::dataObjectList.end()) {
     n += "'";
@@ -495,7 +511,7 @@ KstVectorList::Iterator ey = KST::vectorList.findTag(yErrorVector);
   _doc->setModified();
   _doc->updateDialogs();
 
-return c->tagName();
+  return c->tagName();
 }
 
 
@@ -506,7 +522,7 @@ bool KstIfaceImpl::printImage(const QString& url) {
 
   KstView *view = _app->viewObject();
   view->printToGraphicsFile(url, view->width(), view->height());
-return true; // FIXME: eventually return an error code
+  return true; // FIXME: eventually return an error code
 }
 
 
@@ -522,7 +538,7 @@ bool KstIfaceImpl::printPostScript(const QString& url) {
   printer.setOutputToFile(true);
   printer.setOutputFileName(url);
   view->print(&printer);
-return true; // FIXME: eventually return an error code
+  return true; // FIXME: eventually return an error code
 }
 
 
