@@ -17,26 +17,27 @@
 
 #include "kstplugindialog_i.h"
 
-#include <qwidget.h>
-#include <qstring.h>
 #include <qcheckbox.h>
+#include <qcombobox.h>
 #include <qlabel.h>
+#include <qlayout.h>
 #include <qlineedit.h>
+#include <qlistview.h>
+#include <qobjectlist.h>
 #include <qpushbutton.h>
 #include <qspinbox.h>
-#include <qcombobox.h>
-#include <qobjectlist.h>
-#include <qlistview.h>
+#include <qstring.h>
 #include <qstringlist.h>
-#include <qlayout.h>
+#include <qtimer.h>
+#include <qtooltip.h>
 #include <qwhatsthis.h>
+#include <qwidget.h>
 
-#include <qlineedit.h>
-#include <klocale.h>
-#include <kmessagebox.h>
 #include <kcolorbutton.h>
 #include <kcolordialog.h>
 #include <kdebug.h>
+#include <klocale.h>
+#include <kmessagebox.h>
 #include <kstaticdeleter.h>
 
 #include "plugincollection.h"
@@ -167,43 +168,44 @@ void KstPluginDialogI::update(int new_index) {
     updatePluginList();
   }
 
-  int oldPluginIndex = PluginCombo->currentItem();
   KstPluginPtr plugin = pl[Select->currentItem()];
+  if (!plugin) {
+    return;
+  }
+
+  int oldPluginIndex = PluginCombo->currentItem();
+  int newPluginIndex = _pluginList.findIndex(plugin->plugin()->data()._name);
+  bool updateCombos = (newPluginIndex != oldPluginIndex);
+
+  if (updateCombos) {
+    PluginCombo->setCurrentItem(newPluginIndex);
+    pluginChanged(PluginCombo->currentItem());
+  }
+
   KSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
   if( pPtr != 0 ) {  
-    int newPluginIndex = _pluginList.findIndex(pPtr->data()._name);
-    bool updateCombos = (newPluginIndex != oldPluginIndex);
-
-    if (updateCombos) {
-      PluginCombo->setCurrentItem(newPluginIndex);
-      pluginChanged(PluginCombo->currentItem());
-    }
-
     // Update input vector and scalar combos
     const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
         QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
         assert(field);
-        if( field )
-        {
+        if (field) {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
-          selectedVector = vs->selectedVector( );
+          selectedVector = vs->selectedVector();
           if (!updateCombos) {
             vs->update();
           }
           if (plugin.data() && plugin->isValid()) {
             vs->setSelection(plugin->inputVectors()[(*it)._name]->tagName());
-          }
-          else {
+          } else {
             vs->setSelection(selectedVector);
           }
         }
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
         assert(field);
-        if( field )
-        {
+        if (field) {
           ScalarSelector *ss = static_cast<ScalarSelector*>(field);
           selectedScalar = ss->selectedScalar( );
           if (!updateCombos) {
@@ -211,8 +213,7 @@ void KstPluginDialogI::update(int new_index) {
           }
           if (plugin.data() && plugin->isValid()) {
             ss->setSelection(plugin->inputScalars()[(*it)._name]->tagName());            
-          }
-          else {
+          } else {
             ss->setSelection(selectedScalar);
           }
         }
@@ -226,8 +227,7 @@ void KstPluginDialogI::update(int new_index) {
       for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
         assert(field);
-        if( field )
-        {
+        if (field) {
           QLineEdit *li = static_cast<QLineEdit*>(field);
           if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
             li->setText(plugin->outputVectors()[(*it)._name]->tagName());
@@ -249,8 +249,14 @@ void KstPluginDialogI::update(int new_index) {
     }
   }
 
-  _pluginFrame->resize(_pluginFrame->minimumSizeHint());
-  resize(minimumSizeHint());
+  QTimer::singleShot(0, this, SLOT(fixupLayout()));
+}
+
+
+void KstPluginDialogI::fixupLayout() {
+  _frameWidget->resize(_frameWidget->sizeHint());
+  _pluginFrame->resize(_pluginFrame->sizeHint());
+  resize(sizeHint());
 }
 
 
@@ -284,6 +290,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
     QLineEdit *li = static_cast<QLineEdit*>(field);
 
     if (li->text().isEmpty()) {
+      KMessageBox::sorry(this, i18n("The field %1 is empty.  Not saving.").arg((*it)._name));
       return false;
     }
 
@@ -317,7 +324,8 @@ void KstPluginDialogI::new_I() {
   KstPluginPtr plugin;
   QString tagName = Select->currentText();
 
-  if (KST::dataTagNameNotUnique(tagName)) {
+  if (KST::dataTagNameNotUnique(tagName, true, this)) {
+    Select->setFocus();
     return;
   }
 
@@ -349,7 +357,7 @@ void KstPluginDialogI::new_I() {
   }
   
   if (!plugin || !plugin->isValid()) {
-    KMessageBox::sorry(0L, i18n("There is an error in the plugin you entered. Please fix it."));
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
     return;
   }
 
@@ -372,15 +380,26 @@ void KstPluginDialogI::edit_I() {
   KstPluginPtr plugin = pl[index];
 
   if (!plugin.data()) {
-    KMessageBox::sorry(0L, i18n("The plugin entry has disappeared. Your changes have not been saved."));
+    new_I();
     return;
   }
 
+  int pitem = PluginCombo->currentItem();
+
+  if (tagName.endsWith(newPluginString)) {
+    QString tmpTagName;
+    int i = 0;
+    do {
+      tmpTagName = i18n("P%1-%2").arg(++i).arg(_pluginList[pitem]);
+    } while (KST::dataTagNameNotUnique(tmpTagName, false));
+    tagName = tmpTagName;
+  }
+
   //kdDebug() << "Apply as edit [" << tagName << "]" << endl;
 
   if (tagName != plugin->tagName()) {
-    if (KST::dataTagNameNotUnique(tagName)) {
-      KMessageBox::sorry(0L, i18n("You must choose a unique name for this entry. Your changes have not been saved."));
+    if (KST::dataTagNameNotUnique(tagName, true, this)) {
+      Select->setFocus();
       return;
     }
   }
@@ -390,7 +409,6 @@ void KstPluginDialogI::edit_I() {
   plugin->inputVectors().clear();
   plugin->inputScalars().clear();
 
-  int pitem = PluginCombo->currentItem();
   KSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
 
   // Save the vectors and scalars
@@ -407,11 +425,12 @@ void KstPluginDialogI::edit_I() {
   }
 
   if (!plugin->isValid()) {
-    KMessageBox::sorry(0L, i18n("There is an error in the plugin you entered. Please fix it."));
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
     return;
   }
 
-  plugin = 0L;
+  plugin = 0L; // release
+  pl.clear(); // release
   emit modified();
 }
 
@@ -420,7 +439,7 @@ void KstPluginDialogI::delete_I() {
   QString tag = Select->currentText();
 
   if (tag.isEmpty()) {
-    KMessageBox::sorry(0L, i18n("You need to select an active plugin to delete."));
+    KMessageBox::sorry(this, i18n("You need to select an active plugin to delete."));
   } else {
     KST::dataObjectList.removeTag(tag);
   }
@@ -467,8 +486,14 @@ QString scalarLabelTemplate, vectorLabelTemplate;
 
     if (input) {
       if (scalar) {
-        widget = new ScalarSelector(parent, (*it)._name.latin1());
+        ScalarSelector *w = new ScalarSelector(parent, (*it)._name.latin1());
+        widget = w;
+        connect(w->_scalar, SIGNAL(activated(const QString&)), this, SLOT(updateScalarTooltip(const QString&)));
         connect(widget, SIGNAL(newScalarCreated()), this, SIGNAL(modified()));
+        KstScalarPtr p = *KST::scalarList.findTag(w->_scalar->currentText());
+        if (p) {
+          QToolTip::add(w->_scalar, QString::number(p->value()));
+        }
       } else {
         widget = new VectorSelector(parent, (*it)._name.latin1());
         connect(widget, SIGNAL(newVectorCreated()), this, SIGNAL(modified()));
@@ -518,7 +543,9 @@ void KstPluginDialogI::pluginChanged(int idx) {
     grid->addWidget(new QLabel("Plugin Name:", _frameWidget), 0, 0);
     grid->addWidget(new QLabel(pluginData._name, _frameWidget), 0, 1);
     grid->addWidget(new QLabel("Description:", _frameWidget), 1, 0);
-    grid->addWidget(new QLabel(pluginData._description, _frameWidget), 1, 1);
+    QLabel *lab = new QLabel(pluginData._description, _frameWidget);
+    lab->setAlignment(lab->alignment() | Qt::WordBreak);
+    grid->addWidget(lab, 1, 1);
 
     // Add a separator
     QFrame* line = new QFrame(_frameWidget);
@@ -549,9 +576,7 @@ void KstPluginDialogI::pluginChanged(int idx) {
   _frameWidget->show();
 
   // resize everything
-  _frameWidget->resize(_frameWidget->minimumSizeHint());
-  _pluginFrame->resize(_pluginFrame->minimumSizeHint());
-  resize(minimumSizeHint());
+  QTimer::singleShot(0, this, SLOT(fixupLayout()));
 }
 
 
@@ -562,6 +587,19 @@ void KstPluginDialogI::showPluginManager() {
   updatePluginList();
 }
 
+
+void KstPluginDialogI::updateScalarTooltip(const QString& n) {
+  KstScalarPtr s = *KST::scalarList.findTag(n);
+  QWidget *w = const_cast<QWidget*>(static_cast<const QWidget*>(sender()));
+  if (s) {
+    QToolTip::remove(w);
+    QToolTip::add(w, QString::number(s->value()));
+  } else {
+    QToolTip::remove(w);
+  }
+}
+
+
 #include "kstplugindialog_i.moc"
 
 // vim: ts=2 sw=2 et
