@@ -196,6 +196,7 @@ void KstPsdDialogI::new_I() {
 
   /* verify that the curve name is unique */
   if (KST::dataTagNameNotUnique(tag_name)) {
+    Select->setFocus();
     return;
   }
 
@@ -275,6 +276,7 @@ void KstPsdDialogI::edit_I() {
 
   if (tag_name != curve->tagName()) {
     if (KST::dataTagNameNotUnique(tag_name)) {
+      Select->setFocus();
       return;
     }
   }
@@ -356,3 +358,4 @@ void KstPsdDialogI::delete_I() {
 }
 
 #include "kstpsddialog_i.moc"
+// vim: ts=2 sw=2 et
