@@ -21,8 +21,8 @@
 #include <klocale.h>
 
 /** Create the base scalar */
-KstScalar::KstScalar(const QString& in_tag, double val) : KstObject() {
-  _tag = in_tag;
+KstScalar::KstScalar(const QString& in_tag, double val) : KstObject(), _orphan(false) {
+  QString _tag = in_tag;
   if (_tag.isEmpty()) {
     QString nt = i18n("Anonymous Scalar %1");
     int i = 0;
@@ -35,29 +35,49 @@ KstScalar::KstScalar(const QString& in_tag, double val) : KstObject() {
       _tag += "'";
     }
   }
+  setTagName(_tag);
   _value = val;
   KST::scalarList.append(this);
 }
 
 
+KstScalar::KstScalar(QDomElement& e) : KstObject(), _orphan(false) {
+  QDomNode n = e.firstChild();
+
+  while(!n.isNull()) {
+    QDomElement e = n.toElement();
+    if(!e.isNull()) {
+      if (e.tagName() == "tag") {
+        setTagName(e.text());
+      } else if (e.tagName() == "value") {
+        setValue(e.text().toDouble());
+      }
+    }
+    n = n.nextSibling();
+  }
+  KST::scalarList.append(this);
+}
+
+
 KstScalar::~KstScalar() {
 }
 
 
 KstObject::UpdateType KstScalar::update(int updateCounter) {
-  Q_UNUSED(updateCounter);
-return NO_CHANGE;
+  Q_UNUSED(updateCounter)
+    return NO_CHANGE;
 }
 
 
 void KstScalar::save(QTextStream &ts) {
-  Q_UNUSED(ts);
+  ts << "    <tag>" << tagName() << "</tag>" << endl;
+  ts << "    <value>" << value() << "</value>" << endl;
 }
 
 
 KstScalar& KstScalar::operator=(double v) {
   _value = v;
-return *this;
+  return *this;
 }
 
 
@@ -66,4 +86,11 @@ bool KstScalar::isGlobal() const {
   return KST::scalarList.findTag(tagName()) != KST::scalarList.end();
 }
 
+
+void KstScalar::setValue(double inV) {
+  _value = inV;
+  emit trigger();
+}
+
+#include "kstscalar.moc"
 // vim: et ts=2 sw=2
