@@ -91,7 +91,7 @@ void KstVCurve::commonConstructor(const QString &in_tag, const QColor &in_color)
   _typeString = i18n("Curve");
   NumUsed = 0;
   Color = in_color;
-  _tag = in_tag;
+  setTagName(in_tag);
 }
 
 
@@ -133,11 +133,11 @@ KstObject::UpdateType KstVCurve::update(int update_counter) {
   VY->update(update_counter);
 
   if (hasXError()) {
-    VX->update(update_counter);
+    EX->update(update_counter);
   }
 
   if (hasYError()) {
-    VY->update(update_counter);
+    EY->update(update_counter);
   }
 
   MaxX = VX->max();
@@ -223,7 +223,7 @@ bool KstVCurve::hasYError() const {
 /** Save curve information */
 void KstVCurve::save(QTextStream &ts) {
   ts << " <curve>" << endl;
-  ts << "  <tag>" << _tag << "</tag>" << endl;
+  ts << "  <tag>" << tagName() << "</tag>" << endl;
   ts << "  <xvectag>" << VX->tagName() << "</xvectag>" << endl;
   ts << "  <yvectag>" << VY->tagName() << "</yvectag>" << endl;
   if (EX.data()) {
@@ -272,7 +272,7 @@ KstCurveType KstVCurve::type() const {
 }
 
 QString KstVCurve::propertyString() const {
-  return i18n("%1 vs %2").arg(getXVTag()).arg(getYVTag());
+  return i18n("%1 vs %2").arg(getYVTag()).arg(getXVTag());
 }
 
 void KstVCurve::_showDialog() {
