@@ -37,6 +37,8 @@
 */
 #ifdef isnan
 #define KST_ISNAN(a)	isnan(a)
+#elif defined(__APPLE__)
+#define KST_ISNAN(a)	(a == NAN ? 1 : 0)
 #else
 #define KST_ISNAN(a)	std::isnan(a)
 #endif
@@ -59,7 +61,7 @@ double KST::NOPOINT = 0.0/0.0; // NaN
 KstVector::KstVector(const QString& name, int size)
 : KstObject(), _nsum(0), _scalars(11) {
   //kdDebug() << "+++ CREATING VECTOR: " << (void*) this << endl;
-  _tag = name;
+  KstObject::setTagName(name);
   NumShifted = 0;
   NumNew = 0;
 
@@ -67,26 +69,26 @@ KstVector::KstVector(const QString& name, int size)
     size = INITSIZE;
   }
 
-  if (_tag.isEmpty()) {
+  if (name.isEmpty()) {
     QString nt = "Anonymous Vector %1";
     int i = 0;
     // FIXME: make me more efficient
     do {
-      _tag = nt.arg(i++);
-    } while (KST::vectorTagNameNotUnique(_tag, false));
+      KstObject::setTagName(nt.arg(i++));
+    } while (KST::vectorTagNameNotUnique(tagName(), false));
   } else {
-    while (KST::vectorTagNameNotUnique(_tag, false)) {
-      _tag += "'";
+    while (KST::vectorTagNameNotUnique(tagName(), false)) {
+      KstObject::setTagName(tagName() + "'");
     }
   }
 
   _v = static_cast<double*>(malloc(size*sizeof(double)));
   _size = size;
-  zero();
   _is_rising = false;
 
   CreateScalars();
   KST::vectorList.append(this);
+  zero();
 }
 
 KstVector::~KstVector() {
@@ -97,8 +99,10 @@ KstVector::~KstVector() {
     it.current()->_KShared_unref();
   }
 
-  free(_v);
-  _v = 0;
+  if (_v) {
+    free(_v);
+    _v = 0;
+  }
 }
 
 
@@ -156,36 +160,36 @@ double KstVector::value(int i) {
 }
 
 void KstVector::CreateScalars() {
-  _scalars.insert("max", new KstScalar(_tag + "-Max"));
+  _scalars.insert("max", new KstScalar(tagName() + "-Max"));
   _scalars["max"]->_KShared_ref();
-  _scalars.insert("min", new KstScalar(_tag + "-Min"));
+  _scalars.insert("min", new KstScalar(tagName() + "-Min"));
   _scalars["min"]->_KShared_ref();
-  _scalars.insert("mean", new KstScalar(_tag + "-Mean"));
+  _scalars.insert("mean", new KstScalar(tagName() + "-Mean"));
   _scalars["mean"]->_KShared_ref();
-  _scalars.insert("sigma", new KstScalar(_tag + "-Sigma"));
+  _scalars.insert("sigma", new KstScalar(tagName() + "-Sigma"));
   _scalars["sigma"]->_KShared_ref();
-  _scalars.insert("rms", new KstScalar(_tag + "-Rms"));
+  _scalars.insert("rms", new KstScalar(tagName() + "-Rms"));
   _scalars["rms"]->_KShared_ref();
-  _scalars.insert("ns", new KstScalar(_tag + "-NS"));
+  _scalars.insert("ns", new KstScalar(tagName() + "-NS"));
   _scalars["ns"]->_KShared_ref();
-  _scalars.insert("sum", new KstScalar(_tag + "-Sum"));
+  _scalars.insert("sum", new KstScalar(tagName() + "-Sum"));
   _scalars["sum"]->_KShared_ref();
-  _scalars.insert("sumsquared", new KstScalar(_tag + "-SumSquared"));
+  _scalars.insert("sumsquared", new KstScalar(tagName() + "-SumSquared"));
   _scalars["sumsquared"]->_KShared_ref();
-  _scalars.insert("minpos", new KstScalar(_tag + "-MinPos"));
+  _scalars.insert("minpos", new KstScalar(tagName() + "-MinPos"));
   _scalars["minpos"]->_KShared_ref();
 }
 
 void KstVector::RenameScalars() {
-  _scalars["max"]->setTagName(_tag + "-Max");
-  _scalars["min"]->setTagName(_tag + "-Min");
-  _scalars["mean"]->setTagName(_tag + "-Mean");
-  _scalars["sigma"]->setTagName(_tag + "-Sigma");
-  _scalars["rms"]->setTagName(_tag + "-Rms");
-  _scalars["ns"]->setTagName(_tag + "-NS");
-  _scalars["sum"]->setTagName(_tag + "-Sum");
-  _scalars["sumsquared"]->setTagName(_tag + "-SumSquared");
-  _scalars["minpos"]->setTagName(_tag + "-MinPos");
+  _scalars["max"]->setTagName(tagName() + "-Max");
+  _scalars["min"]->setTagName(tagName() + "-Min");
+  _scalars["mean"]->setTagName(tagName() + "-Mean");
+  _scalars["sigma"]->setTagName(tagName() + "-Sigma");
+  _scalars["rms"]->setTagName(tagName() + "-Rms");
+  _scalars["ns"]->setTagName(tagName() + "-NS");
+  _scalars["sum"]->setTagName(tagName() + "-Sum");
+  _scalars["sumsquared"]->setTagName(tagName() + "-SumSquared");
+  _scalars["minpos"]->setTagName(tagName() + "-MinPos");
 }
 
 void KstVector::UpdateScalars() {
@@ -207,6 +211,7 @@ double* KstVector::realloced(double *memptr, int newSize) {
   double *old = _v;
   _v = memptr;
   _size = newSize;
+  UpdateScalars();
   return old;
 }
 
@@ -215,6 +220,7 @@ void KstVector::zero() {
   for (int i = 0; i < _size; i++) {
     _v[i] = 0.0;
   }
+  UpdateScalars();
 }
 
 
@@ -234,6 +240,7 @@ void KstVector::resize(int sz, bool reinit) {
     }
 #endif
     _size = sz;
+    UpdateScalars();
   }
 }
 
@@ -256,7 +263,7 @@ KstObject::UpdateType KstVector::update(int update_counter) {
     for (i = 0; i < _size && !finite(_v[i]); i++)
       /* Do Nothing */ ;
 
-    if (i==_size) {
+    if (i == _size) {
       _scalars["sum"]->setValue(sum);
       _scalars["sumsquared"]->setValue(sum2);
       _scalars["max"]->setValue(max);
@@ -270,30 +277,31 @@ KstObject::UpdateType KstVector::update(int update_counter) {
 
     i0 = i;
 
-    if (i0>0) {
+    if (i0 > 0) {
       _is_rising = false;
     }
 
     max = min = sum = sum2 = _v[i0];
-    if (_v[i0]>0)
+    if (_v[i0] > 0) {
       minpos = _v[i0];
-    else
+    } else {
       minpos = 1.0E300;
+    }
 
     last_v = _v[i0];
-    for (i=i0; i < _size; i++) {
+    for (i = i0; i < _size; i++) {
       v = _v[i]; // get rid of redirections
 
       if (finite(v)) {
-
         dv = v - last_v;
         dv2 += dv*dv;
 
-        if (v<=last_v) {
-          if (i!=i0) {
+        if (v <= last_v) {
+          if (i != i0) {
             _is_rising = false;
           }
         }
+
         last_v = v;
 
         _nsum++;
@@ -331,8 +339,9 @@ KstObject::UpdateType KstVector::update(int update_counter) {
           last_v = v;
         } else {
           i += 20;
-          if (i < _size)
+          if (i < _size) {
             last_v = _v[i];
+          }
           i++;
         }
       }
@@ -454,4 +463,5 @@ KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QStrin
 return xv;
 }
 
+#include "kstvector.moc"
 // vim: et sw=2 ts=2
