@@ -20,10 +20,11 @@
 #include <stdio.h>
 
 // include files for Qt
-#include <qpainter.h>
-#include <qevent.h>
 #include <qbitmap.h>
 #include <qbrush.h>
+#include <qclipboard.h>
+#include <qevent.h>
+#include <qpainter.h>
 #include <qtabwidget.h>
 
 #include <kdebug.h>
@@ -35,18 +36,20 @@
 #include <kurl.h>
 
 // application specific includes
-#include "kstview.h"
-#include "kstdoc.h"
-#include "kstplotlist.h"
 #include "kst.h"
-#include "kstplotdialog_i.h"
-#include "kstmouse.h"
-#include "kstlabeldialog_i.h"
 #include "kstdatacollection.h"
+#include "kstdoc.h"
+#include "kstlabeldialog_i.h"
+#include "kstmouse.h"
+#include "kstplotdialog_i.h"
+#include "kstplotlist.h"
+#include "kstsettings.h"
+#include "kstview.h"
+
 
 /** setup the view widget */
 KstView::KstView(KstApp *parent, const char *name)
-: QWidget(parent, name) {
+: QWidget(parent, name), _copy_x(KST::NOPOINT), _copy_y(KST::NOPOINT) {
 
   setAcceptDrops(true);
 
@@ -73,7 +76,7 @@ KstView::KstView(KstApp *parent, const char *name)
   _titleId = popupmenu->insertTitle(i18n("Plot"));
   popupmenu->insertItem(i18n("Z&oom"), submenu);
   submenu->insertItem(i18n("Zoom &Maximum"), this, SLOT(zoomMaxSlot()), Key_M);
-  submenu->insertItem(i18n("Zoom Max &Spike Insensivie"),
+  submenu->insertItem(i18n("Zoom Max &Spike Insensitive"),
                       this, SLOT(zoomSpikeInsensiveMaxSlot()), Key_S);
   submenu->insertItem(i18n("Zoom P&revious"), this, SLOT(zoomPrevSlot()), Key_R);
   submenu->insertItem(i18n("Y-Zoom AC Coupled"), this, SLOT(yzoomAcSlot()), Key_A);
@@ -142,9 +145,10 @@ void KstView::printToGraphicsFile(const QString &filename, int in_w, int in_h) {
   double XBorder, maxXBorder = 0;
 
   QPixmap tmpPixmap(in_w, in_h);
-  tmpPixmap.fill();
+  tmpPixmap.fill(KstSettings::globalSettings()->backgroundColor);
 
   QPainter p(&tmpPixmap);
+  p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
 
   v = p.viewport();
 
@@ -256,7 +260,7 @@ void KstView::paintEvent(QPaintEvent *) {
   if (qpixmap == 0L) {
     qpixmap = new QPixmap(width(), height());
     qpixmap->setOptimization(QPixmap::MemoryOptim);
-    qpixmap->fill();
+    qpixmap->fill(KstSettings::globalSettings()->backgroundColor);
     erase();
     return;
   }
@@ -276,7 +280,7 @@ void KstView::paintEvent(QPaintEvent *) {
 
   if (needrecreate) {
     qpixmap->resize(width(), height());
-    qpixmap->fill();
+    qpixmap->fill(KstSettings::globalSettings()->backgroundColor);
 
     // Find max border width
     for (i = i0; i < i1; i++) {
@@ -294,9 +298,11 @@ void KstView::paintEvent(QPaintEvent *) {
         tlx = width() * pl->topLeftX();
       }
       if (i0 == i1) { // only one plot - draw directly onto qpixmap
+        delete p;
         tmpPixmap = qpixmap;
         p = new QPainter(tmpPixmap);
       } else if (i == i0) {
+        delete p;
         tmpPixmap = new QPixmap(int(w),int(h));
         tmpPixmap->setOptimization(QPixmap::MemoryOptim);
         p = new QPainter(tmpPixmap);
@@ -306,6 +312,7 @@ void KstView::paintEvent(QPaintEvent *) {
         p = new QPainter(tmpPixmap);
       }
 
+      p->setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
       XBorder = pl->getXBorder(*p);
       if (XBorder > maxXBorder)
         maxXBorder = XBorder;
@@ -330,6 +337,7 @@ void KstView::paintEvent(QPaintEvent *) {
         tmpPixmap->resize((int)w,(int)h);
         p = new QPainter(tmpPixmap);
       }
+      p->setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
       pl->paint(*p, maxXBorder);
       /* copy the tmppix map to the qpixmap */
       if (i0 != i1) {
@@ -351,6 +359,7 @@ void KstView::paintEvent(QPaintEvent *) {
   // Redraw the mouse box if needed
   if (MouseInfo->getMode() != INACTIVE) {
     QPainter p(this);
+    p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
     if (!MouseInfo->isFirstMove())
       MouseInfo->DrawBox(p);
   }
@@ -370,66 +379,99 @@ void KstView::update() {
 void KstView::updateMouse() {
   int plot = MouseInfo->getPlotNum();
   QString msg;
-  KstPlot* pPlot;
+  KstPlot *pPlot;
 
   if (plot >= 0 && (pPlot = KST::plotList.at(plot))) {
     QRect plot_rect = pPlot->GetPlotRegion();
     QPoint pos = mapFromGlobal(QCursor::pos());
 
     if (!plot_rect.contains(pos)) {
+      _copy_x = KST::NOPOINT;
+      _copy_y = KST::NOPOINT;
       emit newDataMsg(QString::null);
       return;
     }
 
-    double xmin, ymin, xmax, ymax, xpos, ypos;
+    double xmin, ymin, xmax, ymax, xpos, ypos, dx_per_pix;
     pPlot->getLScale(xmin, ymin, xmax, ymax);
 
-    xpos = (double)(pos.x() - plot_rect.left())/(double)plot_rect.width();
-    xpos = xpos * (xmax - xmin) + xmin;
+    // find mouse location in plot units
+    xpos = (double)(pos.x() - plot_rect.left())/(double)plot_rect.width() *
+           (xmax - xmin) + xmin;
     if (pPlot->isXLog()) {
       xpos = pow(10.0, xpos);
     }
 
+    // convert 1 pixel to plot units.
+    dx_per_pix = (double)(pos.x()+2 - plot_rect.left())/
+                 (double)plot_rect.width() *
+                 (xmax - xmin) + xmin;
+    if (pPlot->isXLog()) {
+      dx_per_pix = pow(10.0, dx_per_pix);
+    }
+    dx_per_pix-=xpos;
+
     ypos = (double)(pos.y() - plot_rect.top())/(double)plot_rect.height();
     ypos = ypos * (ymin - ymax) + ymax;
 
     if (pPlot->isYLog()) {
       ypos = pow(10.0, ypos);
     }
- 
-    if (_dataMode) {
-      double newypos = ypos;
-      KstBaseCurvePtr curve;
-      double delta = 9e99;
 
-      for (KstBaseCurveList::Iterator i = pPlot->Curves.begin(); i != pPlot->Curves.end(); ++i) {
-        double xpt = 0.0, ypt = 0.0;
-        if (pPlot->isXLog()) {
-          (*i)->getPoint((*i)->sampleCount() * xpos / ((*i)->maxX() - (*i)->minX()), xpt, ypt);
-        } else {
-          (*i)->getPoint(xpos, xpt, ypt);
-        }
-        if (fabs(ypos - ypt) < delta) {
-          delta = fabs(ypos - ypt);
-          newypos = ypt;
+    if (_dataMode) {
+      double near_x, near_y;
+      double newxpos=0, newypos=0;
+      double d,best_d=1.0E300;
+      int i_near_x;
+      KstBaseCurvePtr curve;
+      msg = "";
+      repaint(false);
+      QPainter p(this);
+
+      for (KstBaseCurveList::Iterator i = pPlot->Curves.begin();
+           i != pPlot->Curves.end(); ++i) {
+        i_near_x = (*i)->getIndexNearXY(xpos, dx_per_pix, ypos);
+        (*i)->getPoint(i_near_x, near_x, near_y);
+        d = fabs(ypos-near_y);
+        if (d < best_d) {
+          newypos = near_y;
+          newxpos = near_x;
+          best_d = d;
           curve = *i;
         }
       }
       if (curve.data()) {
-        msg = i18n("%3 (%1, %2)").arg(xpos).arg(newypos,0,'G').arg(curve->tagName());
+        msg = i18n("%3 (%1, %2)").arg(newxpos).arg(newypos,0,'G').arg(curve->tagName());
+        _copy_x = newxpos;
+        _copy_y = newypos;
+
+        repaint(false);
+        QPainter p(this);
+        pPlot->drawDotAt(p, newxpos, newypos);
         emit newDataMsg(msg);
         return;
       }
+      repaint(false);
+      emit newDataMsg(QString::null);
+      return;
     }
 
     msg = i18n("(%1, %2)").arg(xpos,0,'G').arg(ypos,0,'G');
+    _copy_x = xpos;
+    _copy_y = ypos;
+  } else {
+    _copy_x = KST::NOPOINT;
+    _copy_y = KST::NOPOINT;
   }
+
   emit newDataMsg(msg);
 }
 
 void KstView::mouseMoveEvent(QMouseEvent *e) {
   int x,y;
   int i, i0, iN;
+  int iWidth;
+  int iHeight;
   bool in_plot = false;
   int i_label;
   int plot_num = MouseInfo->getPlotNum();
@@ -442,16 +484,72 @@ void KstView::mouseMoveEvent(QMouseEvent *e) {
     // Start a drag
     KstPlot *plot = KST::plotList.at(plot_num);
     KstLabel *label = plot->labelList.at(i_label);
-    QRect oldExtents = label->extents;
+    QRegion oldExtents = label->extents;
     QPixmap pm(plot->GetWinRegion().width(), plot->GetWinRegion().height());
-    pm.fill();
-    { // Scope is needed to kill off the painter before we resize
+    pm.fill(KstSettings::globalSettings()->backgroundColor);
+    QRect rectBounding = oldExtents.boundingRect( );
+
+    { // Scope is needed to kill off painter before we resize
       QPainter p(&pm);
-      label->draw(p, 2, oldExtents.height() + 2, true);
+      p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
+      label->draw(p, 0, label->v_offset, false);
     }
-    pm.resize(oldExtents.width() + 4, oldExtents.height() + 4);
-    label->extents = oldExtents; // restore them incase the drag is canceled
-    QDragObject *d = new KstLabelDrag(this, plot_num, i_label, _draggablePoint - plot->GetWinRegion().topLeft() - oldExtents.topLeft(), pm);
+
+    //
+    // do not allow the pixmap to be increased in size, else we will be
+    //  drawing a partially uninitialised pixmap during the drag operation...
+    //
+    iWidth = rectBounding.width();
+    if( plot->GetWinRegion().width() < iWidth ) {
+      iWidth = plot->GetWinRegion().width();
+    }
+    iHeight = rectBounding.height();
+    if( plot->GetWinRegion().height() < iHeight ) {
+      iHeight = plot->GetWinRegion().height();
+    }
+
+    pm.resize(iWidth, iHeight);
+    label->extents = oldExtents; // restore them in case the drag is canceled
+    QDragObject *d = new KstLabelDrag(this, plot_num, i_label, _draggablePoint - plot->GetWinRegion().topLeft() - rectBounding.topLeft(), pm);
+    d->dragMove();
+    _draggableLabel = -1;
+    _draggablePlot = -1;
+    return;
+  } else if ( e->state() & Qt::LeftButton &&
+        MouseInfo->getMode() == LABEL_TOOL &&
+        MouseInfo->getLegend(e) &&
+        plot_num == _draggablePlot )
+  {
+    // Start a drag
+    KstPlot *plot = KST::plotList.at(plot_num);
+    KstLegend *legend = plot->Legend;
+    QRegion oldExtents = legend->extents;
+    QPixmap pm(plot->GetWinRegion().width(), plot->GetWinRegion().height());
+    pm.fill(KstSettings::globalSettings()->backgroundColor);
+    QRect rectBounding = oldExtents.boundingRect( );
+
+    { // Scope is needed to kill off painter before we resize
+      QPainter p(&pm);
+      p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
+      legend->draw(&(plot->Curves), p, 0, 0);
+    }
+
+    //
+    // do not allow the pixmap to be increased in size, else we will be
+    //  drawing a partially uninitialised pixmap during the drag operation...
+    //
+    iWidth = rectBounding.width();
+    if( plot->GetWinRegion().width() < iWidth ) {
+      iWidth = plot->GetWinRegion().width();
+    }
+    iHeight = rectBounding.height();
+    if( plot->GetWinRegion().height() < iHeight ) {
+      iHeight = plot->GetWinRegion().height();
+    }
+
+    pm.resize(iWidth, iHeight);
+    legend->extents = oldExtents; // restore them in case the drag is canceled
+    QDragObject *d = new KstLegendDrag(this, plot_num, _draggablePoint - plot->GetWinRegion().topLeft() - rectBounding.topLeft(), pm);
     d->dragMove();
     _draggableLabel = -1;
     _draggablePlot = -1;
@@ -564,10 +662,12 @@ void KstView::mousePressEvent(QMouseEvent *e) {
         updateTieBoxes(this);
         break;
       } else if (plot_rect.contains(e->pos())) {
-        if (_draggableLabel == -1 || MouseInfo->getMode() != LABEL_TOOL) {
-          MouseInfo->mousePressedInPlot(e, plot_rect);
-        } else {
+        if(_draggableLabel >= 0 && MouseInfo->getMode() == LABEL_TOOL) {
           _draggablePlot = i;
+        } else if(_draggableLabel == -1 && MouseInfo->getLegend(e)) {
+          _draggablePlot = i;
+        } else {
+          MouseInfo->mousePressedInPlot(e, plot_rect);
         }
         MouseInfo->setPlotNum(i);
         return;
@@ -663,6 +763,7 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
   if (MouseInfo->getMode() == XY_ZOOMBOX) {
     /* erase the zoombox: */
     QPainter p(this);
+    p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
     if (!MouseInfo->isFirstMove()) MouseInfo->DrawBox(p);
     if ((MouseInfo->getMouseRect().width() >_minMove) &&
         (MouseInfo->getMouseRect().height() >_minMove)) {
@@ -685,7 +786,7 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
       count = KST::plotList.count();
       for (i_plot = 0; i_plot < count; i_plot++) {
         iplot = KST::plotList.at(i_plot);
-        if (plot == iplot || iplot->isTied()) {
+        if (plot == iplot || (iplot->isTied() && plot->isTied())) {
           iplot->setXScaleMode(FIXED);
           iplot->setYScaleMode(FIXED);
           iplot->setLScale(new_xmin, new_ymin, new_xmax, new_ymax);
@@ -696,6 +797,7 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
   } else if (MouseInfo->getMode() == Y_ZOOMBOX) {
     /* erase the zoombox: */
     QPainter p(this);
+    p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
     if (!MouseInfo->isFirstMove()) MouseInfo->DrawBox(p);
     if (MouseInfo->getMouseRect().height() >_minMove) {
 
@@ -710,7 +812,7 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
       count = KST::plotList.count();
       for (i_plot = 0; i_plot < count; i_plot++) {
         iplot = KST::plotList.at(i_plot);
-        if (plot == iplot || iplot->isTied()) {
+        if (plot == iplot || (iplot->isTied() && plot->isTied())) {
           iplot->setYScaleMode(FIXED);
           iplot->setLYScale(new_ymin, new_ymax);
           iplot->pushScale();
@@ -720,6 +822,7 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
   } else   if (MouseInfo->getMode() == X_ZOOMBOX) {
     /* erase the zoombox: */
     QPainter p(this);
+    p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
     if (!MouseInfo->isFirstMove()) MouseInfo->DrawBox(p);
 
     if (MouseInfo->getMouseRect().width() > _minMove) {
@@ -735,7 +838,7 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
       count = KST::plotList.count();
       for (i_plot = 0; i_plot < count; i_plot++) {
         iplot = KST::plotList.at(i_plot);
-        if (plot == iplot || iplot->isTied()) {
+        if (plot == iplot || (iplot->isTied() && plot->isTied())) {
           iplot->setXScaleMode(FIXED);
           iplot->setLXScale(new_xmin, new_xmax);
           iplot->pushScale();
@@ -763,7 +866,8 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
 
 
 void KstView::dragEnterEvent(QDragEnterEvent *e) {
-  e->accept(e->provides("application/x-kst-label-number"));
+  e->accept(e->provides("application/x-kst-label-number") ||
+            e->provides("application/x-kst-legend"));
 }
 
 
@@ -773,6 +877,7 @@ void KstView::dropEvent(QDropEvent *e) {
       QDataStream ds(e->encodedData("application/x-kst-label-number"), IO_ReadOnly);
       int plot, label;
       QPoint hs;
+
       ds >> plot >> label >> hs;
       int posplot = -1, plotCount = KST::plotList.count();
       KstPlot *plotPtr = 0L;
@@ -798,11 +903,47 @@ void KstView::dropEvent(QDropEvent *e) {
         l = plotPtr->labelList.at(label);
       }
 
-      QPoint pos = e->pos() - plotPtr->GetPlotRegion().topLeft() - hs + QPoint(2 /* border offset */, 2 /* border offset */ + 8 /* fudge factor */);
+      QPoint pos = e->pos() - plotPtr->GetWinRegion().topLeft() - hs;
+      QRect rectBounding = l->extents.boundingRect();
+      pos.setX( pos.x() - rectBounding.left() );
+      pos.setY( pos.y() - rectBounding.top() );
       QSize divisor = plotPtr->GetPlotRegion().size();
-      l->move(float(pos.x())/divisor.width(), float(pos.y())/divisor.height());
-      l->extents.setX(pos.x());
-      l->extents.setY(pos.y());
+      l->offsetRelPosition(float(pos.x())/divisor.width(), float(pos.y())/divisor.height());
+      e->accept(true);
+      update();
+      return;
+    }
+  } else if (e->provides("application/x-kst-legend")) {
+    if (e->source() == this) {
+      QDataStream ds(e->encodedData("application/x-kst-legend"), IO_ReadOnly);
+      int plot;
+      QPoint hs;
+
+      ds >> plot >> hs;
+      int posplot = -1, plotCount = KST::plotList.count();
+      KstPlot *plotPtr = 0L;
+      for (int i = 0; i < plotCount; i++) {
+        plotPtr = KST::plotList.at(i);
+        if (plotPtr->GetPlotRegion().contains(e->pos())) {
+          posplot = i;
+          break;
+        }
+      }
+
+      if (posplot == -1 || posplot != plot) {
+        e->accept(false);
+        return;
+      }
+
+      KstLegend *l;
+      l = plotPtr->Legend;
+
+      QPoint pos = e->pos() - plotPtr->GetWinRegion().topLeft() - hs;
+      QRect rectBounding = l->extents.boundingRect();
+      pos.setX( pos.x() - rectBounding.left() );
+      pos.setY( pos.y() - rectBounding.top() );
+      QSize divisor = plotPtr->GetPlotRegion().size();
+      l->offsetRelPosition(float(pos.x())/divisor.width(), float(pos.y())/divisor.height());
       e->accept(true);
       update();
       return;
@@ -1195,6 +1336,7 @@ void KstView::updateTieBox(QPaintDevice *pd, unsigned int i) {
   if (ZoomCurrentPlot) return;
 
   QPainter p(pd);
+  p.setBackgroundColor(KstSettings::globalSettings()->backgroundColor);
   p.setPen(QColor("black"));
 
   plot = KST::plotList.at(i);
@@ -1334,5 +1476,16 @@ void KstView::setDataMode(bool on) {
   updateMouse();
 }
 
+void KstView::copy() {
+  // Don't set the selection because while it does make sense - sort of - it
+  // is far too easy to swipe over Kst while trying to paste a selection
+  // from one window to another.
+
+  // FIXME: we should also provide a custom mime source so that we can
+  //        actually manipulate points of data within Kst.
+  QString msg = i18n("(%1, %2)").arg(_copy_x, 0, 'G').arg(_copy_y, 0, 'G');
+  QApplication::clipboard()->setText(msg);
+}
+
 #include "kstview.moc"
 // vim: ts=2 sw=2 et
