@@ -43,16 +43,23 @@ public:
            const double inScalars[],
            double *outArrays[], int outArrayLens[],
            double outScalars[]) const;
+  int call(const double *const inArrays[], const int inArrayLens[],
+           const double inScalars[],
+           double *outArrays[], int outArrayLens[],
+           double outScalars[], void **local) const;
   int filter(const double *const inArray, int inArrayLen,
              const double inScalars[], double *outArray[],
              int *outArrayLen) const;
+  int filter(const double *const inArray, int inArrayLen,
+             const double inScalars[], double *outArray[],
+             int *outArrayLen, void **local) const;
 
  
   static const int CallError;
 
   class Data {
     public:
-      Data() : _filter(false) {}
+      Data() : _filter(false), _localdata(false) {}
 
       /**
        *   Clear all values.
@@ -89,6 +96,7 @@ public:
 
       // Intro
       bool _filter;
+      bool _localdata;
       QString _name;
       QString _author;
       QString _description;
