@@ -90,6 +90,7 @@ static const QString QS_float = QString::fromLatin1("float");
 static const QString QS_any = QString::fromLatin1("any");
 static const QString QS_integer = QString::fromLatin1("integer");
 static const QString QS_filter = QString::fromLatin1("filter");
+static const QString QS_localdata = QString::fromLatin1("localdata");
 
 
 int PluginXMLParser::parseDOM(const QDomDocument& doc) {
@@ -143,6 +144,8 @@ QDomNode n = element.firstChild();
     QString tn = e.tagName().lower();
     if (tn == QS_modulename) {
       _pluginData._name = e.attribute(QS_name);
+    } else if (tn == QS_localdata) {
+      _pluginData._localdata = true;
     } else if (tn == QS_filter) {
       _pluginData._filter = true;
     } else if (tn == QS_author) {
